#include "runtime.h"
#include "so.h"

#include <dlfcn.h>

static int initlibcurl(void);
static int initlibjpeg(void);
static int initlibv4l2(void);
static int initlibsqlite3(void);
static int initlibXi(void);
static int initlibXss(void);
static int initlibcrypto(void);

int initlib(int lib)
{
   int ret = -1;

   switch(lib) {
      case INIT_LIBCURL:
         ret = initlibcurl();
         break;
      case INIT_LIBJPEG:
         ret = initlibjpeg();
         break;
      case INIT_LIBV4L2:
         ret = initlibv4l2();
         break;
      case INIT_LIBSQLITE3:
         ret = initlibsqlite3();
         break;
      case INIT_LIBXI:
         ret = initlibXi();
         break;
      case INIT_LIBXSS:
         ret = initlibXss();
         break;
      case INIT_LIBCRYPTO:
         ret = initlibcrypto();
         break;
   }

   return ret;
}

static int initlibcurl(void)
{
   int ret = 0;

   do {
      if(libcurl) break;
      ret = -1;
      do {
         if((libcurl = dlopen(so291, RTLD_NOW))) break;
         if((libcurl = dlopen(so292, RTLD_NOW))) break;
      } while(0);
      if(!libcurl) break;
      if(!(*(void **)(&P_curl_global_init) = dlsym(libcurl, so293))) break;
      if(!(*(void **)(&P_curl_easy_init) = dlsym(libcurl, so294))) break;
      if(!(*(void **)(&P_curl_slist_append) = dlsym(libcurl, so295))) break;
      if(!(*(void **)(&P_curl_easy_setopt) = dlsym(libcurl, so296))) break;
      if(!(*(void **)(&P_curl_easy_cleanup) = dlsym(libcurl, so297))) break;
      if(!(*(void **)(&P_curl_slist_free_all) = dlsym(libcurl, so298))) break;
      if(!(*(void **)(&P_curl_easy_perform) = dlsym(libcurl, so299))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_curl_global_init) P_curl_global_init = NULL;
      if(P_curl_easy_init) P_curl_easy_init = NULL;
      if(P_curl_slist_append) P_curl_slist_append = NULL;
      if(P_curl_easy_setopt) P_curl_easy_setopt = NULL;
      if(P_curl_easy_cleanup) P_curl_easy_cleanup = NULL;
      if(P_curl_slist_free_all) P_curl_slist_free_all = NULL;
      if(P_curl_easy_perform) P_curl_easy_perform = NULL;
      if(libcurl) {
         dlclose(libcurl);
         libcurl = NULL;
      }
   }

   return ret;
}

static int initlibjpeg(void)
{
   int ret = 0;

   do {
      if(libjpeg) break;
      ret = -1;
      do {
         if((libjpeg = dlopen(so300, RTLD_NOW))) break;
         if((libjpeg = dlopen(so301, RTLD_NOW))) break;
      } while(0);
      if(!libjpeg) break;
      if(!(*(void **)(&P_jpeg_std_error) = dlsym(libjpeg, so302))) break;
      if(!(*(void **)(&P_jpeg_CreateCompress) = dlsym(libjpeg, so303))) break;
      if(!(*(void **)(&P_jpeg_mem_dest) = dlsym(libjpeg, so304))) break;
      if(!(*(void **)(&P_jpeg_set_defaults) = dlsym(libjpeg, so305))) break;
      if(!(*(void **)(&P_jpeg_set_quality) = dlsym(libjpeg, so306))) break;
      if(!(*(void **)(&P_jpeg_start_compress) = dlsym(libjpeg, so307))) break;
      if(!(*(void **)(&P_jpeg_write_scanlines) = dlsym(libjpeg, so308))) break;
      if(!(*(void **)(&P_jpeg_finish_compress) = dlsym(libjpeg, so309))) break;
      if(!(*(void **)(&P_jpeg_destroy_compress) = dlsym(libjpeg, so310))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_jpeg_std_error) P_jpeg_std_error = NULL;
      if(P_jpeg_CreateCompress) P_jpeg_CreateCompress = NULL;
      if(P_jpeg_mem_dest) P_jpeg_mem_dest = NULL;
      if(P_jpeg_set_defaults) P_jpeg_set_defaults = NULL;
      if(P_jpeg_set_quality) P_jpeg_set_quality = NULL;
      if(P_jpeg_start_compress) P_jpeg_start_compress = NULL;
      if(P_jpeg_write_scanlines) P_jpeg_write_scanlines = NULL;
      if(P_jpeg_finish_compress) P_jpeg_finish_compress = NULL;
      if(P_jpeg_destroy_compress) P_jpeg_destroy_compress = NULL;
      if(libjpeg) {
         dlclose(libjpeg);
         libjpeg = NULL;
      }
   }

   return ret;
}

static int initlibv4l2(void)
{
   int ret = 0;

   do {
      if(libv4l2) break;
      ret = -1;
      do {
         if((libv4l2 = dlopen(so311, RTLD_NOW))) break;
         if((libv4l2 = dlopen(so312, RTLD_NOW))) break;
      } while(0);
      if(!libv4l2) break;
      if(!(*(void **)(&P_v4l2_open) = dlsym(libv4l2, so313))) break;
      if(!(*(void **)(&P_v4l2_close) = dlsym(libv4l2, so314))) break;
      if(!(*(void **)(&P_v4l2_ioctl) = dlsym(libv4l2, so315))) break;
      if(!(*(void **)(&P_v4l2_mmap) = dlsym(libv4l2, so316))) break;
      if(!(*(void **)(&P_v4l2_munmap) = dlsym(libv4l2, so317))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_v4l2_open) P_v4l2_open = NULL;
      if(P_v4l2_close) P_v4l2_open = NULL;
      if(P_v4l2_ioctl) P_v4l2_ioctl = NULL;
      if(P_v4l2_mmap) P_v4l2_mmap = NULL;
      if(P_v4l2_munmap) P_v4l2_munmap = NULL;
      if(libv4l2) {
         dlclose(libv4l2);
         libv4l2 = NULL;
      }
   }

   return ret;
}

static int initlibsqlite3(void)
{
   int ret = 0;

   do {
      if(libsqlite3) break;
      ret = -1;
      do {
         if((libsqlite3 = dlopen(so318, RTLD_NOW))) break;
         if((libsqlite3 = dlopen(so319, RTLD_NOW))) break;
      } while(0);
      if(!libsqlite3) break;
      if(!(*(void **)(&P_sqlite3_open) = dlsym(libsqlite3, so320))) break;
      if(!(*(void **)(&P_sqlite3_exec) = dlsym(libsqlite3, so321))) break;
      if(!(*(void **)(&P_sqlite3_close) = dlsym(libsqlite3, so322))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_sqlite3_open) P_sqlite3_open = NULL;
      if(P_sqlite3_exec) P_sqlite3_exec = NULL;
      if(P_sqlite3_close) P_sqlite3_close = NULL;
      if(libsqlite3) {
         dlclose(libsqlite3);
         libsqlite3 = NULL;
      }
   }

   return ret;
}

static int initlibXi(void)
{
   int ret = 0;

   do {
      if(libXi) break;
      ret = -1;
      do {
         if((libXi = dlopen(so323, RTLD_NOW))) break;
         if((libXi = dlopen(so324, RTLD_NOW))) break;
      } while(0);
      if(!libXi) break;
      if(!(*(void **)(&P_XOpenDevice) = dlsym(libXi, so325))) break;
      if(!(*(void **)(&P_XListInputDevices) = dlsym(libXi, so326))) break;
      if(!(*(void **)(&P_XSelectExtensionEvent) = dlsym(libXi, so327))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_XOpenDevice) P_XOpenDevice = NULL;
      if(P_XListInputDevices) P_XListInputDevices = NULL;
      if(P_XSelectExtensionEvent) P_XSelectExtensionEvent = NULL;
      if(libXi) {
         dlclose(libXi);
         libXi = NULL;
      }
   }

   return ret;
}

static int initlibXss(void)
{
   int ret = 0;

   do {
      if(libXss) break;
      ret = -1;
      do {
         if((libXss = dlopen(so328, RTLD_NOW))) break;
         if((libXss = dlopen(so329, RTLD_NOW))) break;
      } while(0);
      if(!libXss) break;
      if(!(*(void **)(&P_XScreenSaverQueryExtension) = dlsym(libXss, so330))) break;
      if(!(*(void **)(&P_XScreenSaverQueryInfo) = dlsym(libXss, so331))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_XScreenSaverQueryExtension) P_XScreenSaverQueryExtension = NULL;
      if(P_XScreenSaverQueryInfo) P_XScreenSaverQueryInfo = NULL;
      if(libXss) {
         dlclose(libXss);
         libXss = NULL;
      }
   }

   return ret;
}

static int initlibcrypto(void)
{
   int ret = 0;

   do {
      if(libcrypto) break;
      ret = -1;
      do {
         if((libcrypto = dlopen(so332, RTLD_NOW))) break;
         if((libcrypto = dlopen(so333, RTLD_NOW))) break;
         if((libcrypto = dlopen(so334, RTLD_NOW))) break;
      } while(0);
      if(!libcrypto) break;
      if(!(*(void **)(&P_BIO_ctrl) = dlsym(libcrypto, so335))) break;
      if(!(*(void **)(&P_BIO_f_cipher) = dlsym(libcrypto, so336))) break;
      if(!(*(void **)(&P_BIO_f_md) = dlsym(libcrypto, so337))) break;
      if(!(*(void **)(&P_BIO_free) = dlsym(libcrypto, so338))) break;
      if(!(*(void **)(&P_BIO_free_all) = dlsym(libcrypto, so339))) break;
      if(!(*(void **)(&P_BIO_gets) = dlsym(libcrypto, so340))) break;
      if(!(*(void **)(&P_BIO_new) = dlsym(libcrypto, so341))) break;
      if(!(*(void **)(&P_BIO_new_file) = dlsym(libcrypto, so342))) break;
      if(!(*(void **)(&P_BIO_new_fp) = dlsym(libcrypto, so343))) break;
      if(!(*(void **)(&P_BIO_pop) = dlsym(libcrypto, so344))) break;
      if(!(*(void **)(&P_BIO_printf) = dlsym(libcrypto, so345))) break;
      if(!(*(void **)(&P_BIO_push) = dlsym(libcrypto, so346))) break;
      if(!(*(void **)(&P_BIO_read) = dlsym(libcrypto, so347))) break;
      if(!(*(void **)(&P_BIO_set_cipher) = dlsym(libcrypto, so348))) break;
      if(!(*(void **)(&P_BIO_s_mem) = dlsym(libcrypto, so349))) break;
      if(!(*(void **)(&P_BIO_s_null) = dlsym(libcrypto, so350))) break;
      if(!(*(void **)(&P_BIO_write) = dlsym(libcrypto, so351))) break;
      if(!(*(void **)(&P_EVP_CIPHER_CTX_set_padding) = dlsym(libcrypto, so352))) break;
      if(!(*(void **)(&P_EVP_get_cipherbyname) = dlsym(libcrypto, so353))) break;
      if(!(*(void **)(&P_EVP_sha1) = dlsym(libcrypto, so354))) break;
      if(!(*(void **)(&P_MD5) = dlsym(libcrypto, so355))) break;
      if(!(*(void **)(&P_OpenSSL_add_all_ciphers) = dlsym(libcrypto, so356))) break;
      if(!(*(void **)(&P_RAND_pseudo_bytes) = dlsym(libcrypto, so357))) break;
      if(!(*(void **)(&P_SHA1_Final) = dlsym(libcrypto, so358))) break;
      if(!(*(void **)(&P_SHA1_Init) = dlsym(libcrypto, so359))) break;
      if(!(*(void **)(&P_SHA1_Update) = dlsym(libcrypto, so360))) break;
      ret = 0;
   } while(0);

   if(ret == -1) {
      if(P_BIO_ctrl) P_BIO_ctrl = NULL;
      if(P_BIO_f_cipher) P_BIO_f_cipher = NULL;
      if(P_BIO_f_md) P_BIO_f_md = NULL;
      if(P_BIO_free) P_BIO_free = NULL;
      if(P_BIO_free_all) P_BIO_free_all = NULL;
      if(P_BIO_gets) P_BIO_gets = NULL;
      if(P_BIO_new) P_BIO_new = NULL;
      if(P_BIO_new_file) P_BIO_new_file = NULL;
      if(P_BIO_new_fp) P_BIO_new_fp = NULL;
      if(P_BIO_pop) P_BIO_pop = NULL;
      if(P_BIO_printf) P_BIO_printf = NULL;
      if(P_BIO_push) P_BIO_push = NULL;
      if(P_BIO_read) P_BIO_read = NULL;
      if(P_BIO_set_cipher) P_BIO_set_cipher = NULL;
      if(P_BIO_s_mem) P_BIO_s_mem = NULL;
      if(P_BIO_s_null) P_BIO_s_null = NULL;
      if(P_BIO_write) P_BIO_write = NULL;
      if(P_EVP_CIPHER_CTX_set_padding) P_EVP_CIPHER_CTX_set_padding = NULL;
      if(P_EVP_get_cipherbyname) P_EVP_get_cipherbyname = NULL;
      if(P_EVP_sha1) P_EVP_sha1 = NULL;
      if(P_MD5) P_MD5 = NULL;
      if(P_OpenSSL_add_all_ciphers) P_OpenSSL_add_all_ciphers = NULL;
      if(P_RAND_pseudo_bytes) P_RAND_pseudo_bytes = NULL;
      if(P_SHA1_Final) P_SHA1_Final = NULL;
      if(P_SHA1_Init) P_SHA1_Init = NULL;
      if(P_SHA1_Update) P_SHA1_Update = NULL;
      if(libcrypto) {
         dlclose(libcrypto);
         libcrypto = NULL;
      }
   }

   return ret;
}
